<?php

namespace Drupal\visitors\Controller\Report;

use Drupal\Core\Datetime\DateFormatterInterface;
use Drupal\Core\Form\FormBuilderInterface;
use Drupal\Core\StringTranslation\TranslationInterface;
use Drupal\visitors\VisitorsReportInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Hour Report controller.
 */
final class TimeController extends ReportController {
  /**
   * The date service.
   *
   * @var \Drupal\Core\Datetime\DateFormatterInterface
   */
  protected $date;

  /**
   * The form builder service.
   *
   * @var \Drupal\Core\Form\FormBuilderInterface
   */
  protected $formBuilder;

  /**
   * The report service.
   *
   * @var \Drupal\visitors\VisitorsReportInterface
   */
  protected $report;

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container): TimeController {
    return new self(
      $container->get('date.formatter'),
      $container->get('form_builder'),
      $container->get('visitors.report'),
      $container->get('string_translation')
    );
  }

  /**
   * Constructs a Hours object.
   *
   * @param \Drupal\Core\Datetime\DateFormatterInterface $date_formatter
   *   The date service.
   * @param \Drupal\Core\Form\FormBuilderInterface $form_builder
   *   The form builder service.
   * @param \Drupal\visitors\VisitorsReportInterface $report_service
   *   The report service.
   * @param \Drupal\Core\StringTranslation\TranslationInterface $string_translation
   *   The string translation service.
   */
  public function __construct(DateFormatterInterface $date_formatter, FormBuilderInterface $form_builder, VisitorsReportInterface $report_service, TranslationInterface $string_translation) {

    $this->date        = $date_formatter;
    $this->formBuilder = $form_builder;
    $this->report      = $report_service;
    $this->setStringTranslation($string_translation);

  }

  /**
   * Returns a hours page.
   *
   * @return array
   *   A render array representing the hours page content.
   */
  public function display(): array {
    $form = $this->formBuilder->getForm('Drupal\visitors\Form\DateFilter');

    $first_row['local_header'] = [
      '#prefix'   => '<div class="layout-column--half">',
      '#markup' => '<h2>' . $this->t('Local time') . '</h2>',
      '#suffix'   => '</div>',
    ];
    $first_row['hour_header'] = [
      '#prefix'   => '<div class="layout-column--half">',
      '#markup' => '<h2>' . $this->t('Hours') . '</h2>',
      '#suffix'   => '</div>',
    ];

    $second_row['local_chart'] = $this->localHoursReportChart();
    $second_row['hour_chart'] = $this->hoursReportChart();

    $third_row['local_table'] = $this->localHoursReportTable();
    $third_row['hour_table'] = $this->hoursReportTable();

    $fourth_row['week_header'] = [
      '#prefix'   => '<div class="layout-column--half">',
      '#markup' => '<h2>' . $this->t('Day of Week') . '</h2>',
      '#suffix'   => '</div>',
    ];
    $fourth_row['month_header'] = [
      '#prefix'   => '<div class="layout-column--half">',
      '#markup' => '<h2>' . $this->t('Day of Month') . '</h2>',
      '#suffix'   => '</div>',
    ];

    $fifth_row['day_chart'] = $this->dayOfWeekReportChart();
    $fifth_row['month_chart'] = $this->dayOfMonthReportChart();

    $sixth_row['week_table'] = $this->dayOfWeekReportTable();
    $sixth_row['hour_table'] = $this->dayOfMonthReportTable();

    $seventh_row['month_header'] = [
      '#prefix'   => '<div class="layout-column--half">',
      '#markup' => '<h2>' . $this->t('Monthly') . '</h2>',
      '#suffix'   => '</div>',
    ];
    $eighth_row['month_chart'] = $this->monthlyReportChart();
    $ninth_row['month_table'] = $this->monthlyReportTable();

    return [
      'visitors_date_filter_form' => $form,

      'main' => [
        '#type' => 'container',
        '#attributes' => [
          'class' => ['visitors-main'],
        ],
        '1' => [
          '#prefix'   => '<div class="layout-row">',
          'blocks'    => $first_row,
          '#suffix'   => '</div>',
        ],
        '2' => [
          '#prefix'   => '<div class="layout-row">',
          'blocks'    => $second_row,
          '#suffix'   => '</div>',
        ],
        '3' => [
          '#prefix'   => '<div class="layout-row">',
          'blocks'    => $third_row,
          '#suffix'   => '</div>',
        ],
        '4' => [
          '#prefix'   => '<div class="layout-row">',
          'blocks'    => $fourth_row,
          '#suffix'   => '</div>',
        ],
        '5' => [
          '#prefix'   => '<div class="layout-row">',
          'blocks'    => $fifth_row,
          '#suffix'   => '</div>',
        ],
        '6' => [
          '#prefix'   => '<div class="layout-row">',
          'blocks'    => $sixth_row,
          '#suffix'   => '</div>',
        ],
        '7' => [
          '#prefix'   => '<div class="layout-row">',
          'blocks'    => $seventh_row,
          '#suffix'   => '</div>',
        ],
        '8' => [
          '#prefix'   => '<div class="layout-row">',
          'blocks'    => $eighth_row,
          '#suffix'   => '</div>',
        ],
        '9' => [
          '#prefix'   => '<div class="layout-row">',
          'blocks'    => $ninth_row,
          '#suffix'   => '</div>',
        ],
      ],
      '#attached' => [
        'library' => [
          'visitors/visitors.report',
        ],
      ],
    ];
  }

  /**
   * Returns a table header for hours reports.
   *
   * @return array
   *   A render array representing the table header info.
   */
  protected function getHourHeader(): array {
    return [
      'hour' => [
        'data'      => $this->t('Hour'),
        'field'     => 'hour',
        'specifier' => 'hour',
        'class'     => [RESPONSIVE_PRIORITY_LOW],
        'sort'      => 'asc',
      ],
      'count' => [
        'data'      => $this->t('Unique Visitors'),
        'field'     => 'count',
        'specifier' => 'count',
        'class'     => [RESPONSIVE_PRIORITY_LOW, 'text-align-right'],
      ],
    ];
  }

  /**
   * Returns a table header for day of week report.
   *
   * @return array
   *   A render array representing the table header info.
   */
  protected function getDayWeekHeader(): array {
    return [
      'day' => [
        'data' => $this->t('Day'),
        'class'     => [RESPONSIVE_PRIORITY_LOW],
      ],
      'count' => [
        'data' => $this->t('Unique Visitors'),
        'class'     => [RESPONSIVE_PRIORITY_LOW, 'text-align-right'],
      ],
    ];
  }

  /**
   * Returns a chart for hours report.
   */
  protected function hoursReportChart() {

    $header = $this->getHourHeader();

    $y = [];
    for ($i = 0; $i < 24; $i++) {
      $y[$i] = 0;
    }
    $results = $this->report->hours();
    foreach ($results as $data) {
      $y[(int) $data[0]] = $data[1];
    }

    return [
      '#chart_id' => 'hours',
      '#theme'  => 'visitors_jqplot',
      '#x'      => implode(', ', range(0, 23)),
      '#y'      => implode(', ', $y),
      '#width'  => $this->report->width(),
      '#height' => $this->report->height(),
      '#attributes'   => [
        'class' => ['layout-column--half'],
      ],
    ];
  }

  /**
   * Returns a table for hours report.
   */
  protected function hoursReportTable() {

    $header = $this->getHourHeader();
    $data = $this->report->hours();
    $rows = [];
    foreach ($data as $d) {
      $rows[] = [
        'data' => [
          ['data' => $d[0]],
          ['data' => $d[1], 'class' => ['text-align-right']],
        ],
      ];
    }
    return [
      '#type'  => 'table',
      '#header' => $header,
      '#rows'   => $rows,
      '#attributes'   => [
        'class' => ['layout-column--half'],
      ],
    ];
  }

  /**
   * Returns a table header for day of month report.
   */
  protected function localHoursReportChart() {

    $header = $this->getHourHeader();

    $y = [];
    for ($i = 0; $i < 24; $i++) {
      $y[$i] = 0;
    }
    $results = $this->report->localHours();
    foreach ($results as $data) {
      $y[(int) $data[0]] = $data[1];
    }

    return [
      '#chart_id' => 'local_hours',
      '#theme'  => 'visitors_jqplot',
      '#x'      => implode(', ', range(0, 23)),
      '#y'      => implode(', ', $y),
      '#width'  => $this->report->width(),
      '#height' => $this->report->height(),
      '#attributes'   => [
        'class' => ['layout-column--half'],
      ],
    ];
  }

  /**
   * Returns a table for local hours report.
   */
  protected function localHoursReportTable() {

    $header = $this->getHourHeader();
    $data = $this->report->localHours($header);
    $rows = [];
    foreach ($data as $d) {
      $rows[] = [
        'data' => [
          ['data' => $d[0]],
          ['data' => $d[1], 'class' => ['text-align-right']],
        ],
      ];
    }

    return [
      '#type'  => 'table',
      '#header' => $header,
      '#rows'   => $rows,
      '#attributes'   => [
        'class' => ['layout-column--half'],
      ],
    ];
  }

  /**
   * Returns a chart for day of week report.
   */
  protected function dayOfWeekReportChart() {

    $header  = $this->getDayWeekHeader();
    $results = $this->report->daysOfWeek();
    $days    = $this->report->getTranslatedDays();
    $x       = [];
    $y       = [];

    foreach ($days as $translated) {
      $key = $translated->render();
      $x[] = '"' . $key . '"';
      $y[$key] = 0;
    }

    foreach ($results as $data) {
      $y[$data[0]] = $data[1];
    }
    return [
      '#chart_id' => 'day_week',
      '#theme'  => 'visitors_jqplot',
      '#x'      => implode(', ', $x),
      '#y'      => implode(', ', $y),
      '#width'  => $this->report->width(),
      '#height' => $this->report->height(),
      '#attributes'   => [
        'class' => ['layout-column--half'],
      ],
    ];
  }

  /**
   * Returns a table for day of week report.
   */
  protected function dayOfWeekReportTable() {

    $header = $this->getDayWeekHeader();
    $data = $this->report->daysOfWeek();
    $rows = [];
    foreach ($data as $d) {
      $rows[] = [
        'data' => [
          ['data' => $d[0]],
          ['data' => $d[1], 'class' => ['text-align-right']],
        ],
      ];
    }
    return [
      '#type'  => 'table',
      '#header' => $header,
      '#rows'   => $rows,
      '#attributes'   => [
        'class' => ['layout-column--half'],
      ],
    ];
  }

  /**
   * Returns a table header for day of month report.
   */
  protected function getDayMonthHeader(): array {
    return [
      'day' => [
        'data'      => $this->t('Day'),
        'field'     => 'day',
        'specifier' => 'day',
        'class'     => [RESPONSIVE_PRIORITY_LOW],
        'sort'      => 'asc',
      ],
      'count' => [
        'data'      => $this->t('Unique Visitors'),
        'field'     => 'count',
        'specifier' => 'count',
        'class'     => [RESPONSIVE_PRIORITY_LOW, 'text-align-right'],
      ],
    ];
  }

  /**
   * Returns a table for day of month report.
   */
  protected function dayOfMonthReportTable() {

    $header = $this->getDayMonthHeader();
    $data = $this->report->daysOfMonth();
    $rows = [];
    foreach ($data as $d) {
      $rows[] = [
        'data' => [
          ['data' => $d[0]],
          ['data' => $d[1], 'class' => ['text-align-right']],
        ],
      ];
    }

    return [
      '#type'  => 'table',
      '#header' => $header,
      '#rows'   => $rows,
      '#attributes'   => [
        'class' => ['layout-column--half'],
      ],
    ];
  }

  /**
   * Returns a chart for day of month report.
   */
  protected function dayOfMonthReportChart() {
    $header  = $this->getDayMonthHeader();
    $results = $this->report->daysOfMonth();

    $y = [];
    for ($i = 1; $i <= 31; $i++) {
      $y[$i] = 0;
    }

    foreach ($results as $data) {
      $y[$data[0]] = $data[1];
    }

    return [
      '#chart_id' => 'day_month',
      '#theme'  => 'visitors_jqplot',
      '#x'      => implode(', ', range(1, 31)),
      '#y'      => implode(', ', $y),
      '#width'  => $this->report->width(),
      '#height' => $this->report->height(),
      '#attributes'   => [
        'class' => ['layout-column--half'],
      ],
    ];
  }

  /**
   * Returns a chart for monthly report.
   */
  protected function monthlyReportChart() {
    $header  = $this->getMonthlyHeader();
    $results = $this->report->monthly();
    $x       = [];
    $y       = [];

    foreach ($results as $data) {
      $x[] = "\"$data[0]\"";
      $y[$data[0]] = $data[1];
    }

    return [
      '#chart_id' => 'monthly',
      '#theme'  => 'visitors_jqplot',
      '#x'      => implode(', ', $x),
      '#y'      => implode(', ', $y),
      '#width'  => $this->report->width(),
      '#height' => $this->report->height(),
      '#attributes'   => [
        'class' => ['layout-column--half'],
      ],
    ];

  }

  /**
   * Returns a table for monthly report.
   */
  protected function monthlyReportTable() {
    $header = $this->getMonthlyHeader();
    $data = $this->report->monthly();
    $rows = [];
    foreach ($data as $d) {
      $rows[] = [
        'data' => [
          ['data' => $d[0]],
          ['data' => $d[1], 'class' => ['text-align-right']],
        ],
      ];
    }

    return [
      '#type'  => 'table',
      '#header' => $header,
      '#rows'   => $rows,
      '#attributes'   => [
        'class' => ['layout-column--half'],
      ],
    ];
  }

  /**
   * Returns a table header configuration.
   *
   * @return array
   *   A render array representing the table header info.
   */
  protected function getMonthlyHeader(): array {
    return [
      'm' => [
        'data'      => $this->t('Month'),
        'field'     => 'm',
        'specifier' => 'm',
        'class'     => [RESPONSIVE_PRIORITY_LOW],
        'sort'      => 'asc',
      ],
      'count' => [
        'data'      => $this->t('Unique Visitors'),
        'field'     => 'count',
        'specifier' => 'count',
        'class'     => [RESPONSIVE_PRIORITY_LOW, 'text-align-right'],
      ],
    ];
  }

}
